import * as i0 from '@angular/core';
import { EventEmitter, Component, ViewChild, Input, Output, HostListener, NgModule } from '@angular/core';

class PopoutWindowComponent {
    constructor(renderer2, elementRef) {
        this.renderer2 = renderer2;
        this.elementRef = elementRef;
        this.suppressCloneStyles = false;
        this.closed = new EventEmitter();
        this.isOut = false;
    }
    get isPoppedOut() {
        return this.isOut;
    }
    beforeunloadHandler() {
        this.close();
    }
    ngOnDestroy() {
        this.observer?.disconnect();
        this.close();
    }
    close() {
        if (this.popoutWindow) {
            this.popoutWindow.close();
            this.popoutWindow = null;
            this.isOut = false;
            this.closed.next(true);
        }
    }
    popIn() {
        this.renderer2.appendChild(this.elementRef.nativeElement, this.innerWrapper.nativeElement);
        this.close();
    }
    popOut() {
        if (!this.popoutWindow) {
            const elmRect = this.innerWrapper.nativeElement.getBoundingClientRect();
            const navHeight = window.outerHeight - window.innerHeight;
            const navWidth = window.outerWidth - window.innerWidth;
            const winLeft = this.windowLeft || window.screenX + navWidth + elmRect.left;
            const winTop = this.windowTop || window.screenY + navHeight + elmRect.top - 60;
            this.popoutWindow = window.open('', `popoutWindow${Date.now()}`, `width=${this.windowWidth > 99 ? this.windowWidth : elmRect.width},
        height=${this.windowHeight > 99 ? this.windowHeight : elmRect.height + 1},
        left=${winLeft},
        top=${winTop}`);
            this.popoutWindow.document.title = this.windowTitle ? this.windowTitle : window.document.title;
            this.popoutWindow.document.body.style.margin = '0';
            if (!this.suppressCloneStyles) {
                document.head.querySelectorAll('style').forEach(node => {
                    this.popoutWindow.document.head.appendChild(node.cloneNode(true));
                });
                this.observeStyleChanges();
                document.head.querySelectorAll('link[rel="stylesheet"]').forEach(node => {
                    this.popoutWindow.document.head.insertAdjacentHTML('beforeend', `<link rel="stylesheet" type="${node.type}" href="${node.href}">`);
                });
                document.fonts.forEach(node => {
                    this.popoutWindow.document.fonts.add(node);
                });
            }
            if (this.windowStyleUrl) {
                this.popoutWindow.document.head.insertAdjacentHTML('beforeend', `<link rel="stylesheet" type="text/css" href="${window.location.origin}/${this.windowStyleUrl}">`);
            }
            if (this.windowStyle) {
                this.popoutWindow.document.head.insertAdjacentHTML('beforeend', `<style>${this.windowStyle}</style>`);
            }
            this.renderer2.appendChild(this.popoutWindow.document.body, this.innerWrapper.nativeElement);
            this.isOut = true;
            this.popoutWindow.addEventListener('unload', () => {
                this.popIn();
            });
        }
        else {
            this.popoutWindow.focus();
        }
    }
    observeStyleChanges() {
        const docHead = document.querySelector('head');
        this.observer?.disconnect();
        this.observer = new MutationObserver((mutations) => {
            mutations.forEach((mutation) => {
                mutation.addedNodes.forEach((node) => {
                    if (node.nodeName === 'STYLE') {
                        this.popoutWindow.document.head.appendChild(node.cloneNode(true));
                    }
                });
            });
        });
        this.observer.observe(docHead, { childList: true });
    }
}
PopoutWindowComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PopoutWindowComponent, deps: [{ token: i0.Renderer2 }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Component });
PopoutWindowComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.2.12", type: PopoutWindowComponent, selector: "popout-window", inputs: { windowWidth: "windowWidth", windowHeight: "windowHeight", windowLeft: "windowLeft", windowTop: "windowTop", windowTitle: "windowTitle", windowStyle: "windowStyle", windowStyleUrl: "windowStyleUrl", suppressCloneStyles: "suppressCloneStyles", isPoppedOut: "isPoppedOut" }, outputs: { closed: "closed" }, host: { listeners: { "window:beforeunload": "beforeunloadHandler()" } }, viewQueries: [{ propertyName: "innerWrapper", first: true, predicate: ["innerWrapper"], descendants: true }], ngImport: i0, template: `
    <div #innerWrapper style="width: 100%; height: 100%; overflow: auto;">
      <ng-content></ng-content>
    </div>
  `, isInline: true });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PopoutWindowComponent, decorators: [{
            type: Component,
            args: [{
                    selector: 'popout-window',
                    template: `
    <div #innerWrapper style="width: 100%; height: 100%; overflow: auto;">
      <ng-content></ng-content>
    </div>
  `
                }]
        }], ctorParameters: function () { return [{ type: i0.Renderer2 }, { type: i0.ElementRef }]; }, propDecorators: { innerWrapper: [{
                type: ViewChild,
                args: ['innerWrapper']
            }], windowWidth: [{
                type: Input
            }], windowHeight: [{
                type: Input
            }], windowLeft: [{
                type: Input
            }], windowTop: [{
                type: Input
            }], windowTitle: [{
                type: Input
            }], windowStyle: [{
                type: Input
            }], windowStyleUrl: [{
                type: Input
            }], suppressCloneStyles: [{
                type: Input
            }], isPoppedOut: [{
                type: Input
            }], closed: [{
                type: Output
            }], beforeunloadHandler: [{
                type: HostListener,
                args: ['window:beforeunload']
            }] } });

class PopoutWindowModule {
}
PopoutWindowModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PopoutWindowModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
PopoutWindowModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.2.12", ngImport: i0, type: PopoutWindowModule, declarations: [PopoutWindowComponent], exports: [PopoutWindowComponent] });
PopoutWindowModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PopoutWindowModule });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.2.12", ngImport: i0, type: PopoutWindowModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [PopoutWindowComponent],
                    exports: [PopoutWindowComponent]
                }]
        }] });

/*
 * Public API Surface of popout-window
 */

/**
 * Generated bundle index. Do not edit.
 */

export { PopoutWindowComponent, PopoutWindowModule };
//# sourceMappingURL=angular-popout-window.mjs.map
